package com.winhc.bigdata.udf;

import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.aliyun.odps.Column;
import com.aliyun.odps.udf.ExecutionContext;
import com.aliyun.odps.udf.UDF;
import com.winhc.bigdata.bean.BaseFieldMeta;
import org.apache.commons.lang3.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author: π
 * @Description: 转sql文件
 */
public class DataToStrV3 extends UDF {

    private static Map<String, List<BaseFieldMeta>> meta_map = new HashMap<>();

    private static String API_URL = "http://192.168.2.65/api/out/findAll?customers_id=base";

    static {
        if (isWindows()) {
            API_URL = "http://whc-data-dict.106.15.78.184.myip.loc/api/out/findAll?customers_id=base";
        }
    }

    public static Boolean isWindows() {
        return System.getProperty("os.name").contains("Windows");
    }

    @Override
    public void setup(ExecutionContext ctx) {
        HttpRequest get = HttpUtil.createGet(API_URL);
        get.header("sign", "07014b4cc0ec4938625c9a5d67e76b61");
        get.header("Host", "whc-data-dict.192.168.2.65.myip.loc");
        String res = get.execute().body();
        List<BaseFieldMeta> baseFieldMetas = JSON.parseArray(res, BaseFieldMeta.class);
        meta_map = baseFieldMetas.stream().filter(BaseFieldMeta::getVisible_to_customers).collect(Collectors.groupingBy(BaseFieldMeta::getTn));
    }


    public String evaluate(List<String> text_list, String tn) {
        if (null == text_list || text_list.isEmpty()) return null;
        List<JSONObject> objectList = text_list.stream().map(JSON::parseObject).collect(Collectors.toList());

        if (!meta_map.containsKey(tn)) {
            return null;
        }
        List<JSONObject> out_list = objectList.stream().map(x -> pre2Data(x, tn)).collect(Collectors.toList());
        List<BaseFieldMeta> cols = pre2ColsSort(tn);

        List<String> rowkeys = Collections.singletonList("rowkey");
        if (tn.equals("company")) {
            rowkeys = Collections.singletonList("company_id");
        }
        //转化sql --注意排序
        String sql = getUpsertSqlBatch(tn, cols, rowkeys, out_list) + ";";

        return sql;
    }

    public String evaluate(String text, String tn) {
        if (null == text) return null;
        JSONObject data = JSON.parseObject(text);

        if (!meta_map.containsKey(tn)) {
            return null;
        }
        JSONObject out = pre2Data(data, tn);
        List<BaseFieldMeta> cols = pre2ColsSort(tn);

        List<String> rowkeys = Collections.singletonList("rowkey");
        if (tn.equals("company")) {
            rowkeys = Collections.singletonList("company_id");
        }
        //转化sql --注意排序
        String sql = getUpsertSql(tn, cols, rowkeys, out) + ";";

        return sql;
    }

    /**
     * 转化数据
     *
     * @param data
     * @param tn
     * @return
     */
    public JSONObject pre2Data(JSONObject data, String tn) {

        JSONObject out = new JSONObject();
        List<BaseFieldMeta> list = getMetaList(tn);

        //转化数据类型
        list.forEach(c -> {
            String name = c.getName();
            String typeName = c.getOdps_type();
            Object value = getValueOdps(data, name, typeName);
            out.put(name, value);
        });

        return out;
    }

    /**
     * 字段排序
     *
     * @param tn
     * @return
     */
    public List<BaseFieldMeta> pre2ColsSort(String tn) {
        List<BaseFieldMeta> list = getMetaList(tn);
        //字段类型和顺序
        return list.stream().sorted(Comparator.comparing(BaseFieldMeta::getField_position)).collect(Collectors.toList());
    }

    private static List<BaseFieldMeta> getMetaList(String tn) {
        return meta_map.get(tn).stream()
                .filter(BaseFieldMeta::getVisible_to_customers)
                .collect(Collectors.toList());
    }

    /**
     * json 转化 sql batch
     *
     * @param tableName   表名
     * @param cols        字段值
     * @param primaryKeys 主键
     * @param data_list   数据
     * @return
     */
    public static String getUpsertSqlBatch(String tableName, List<BaseFieldMeta> cols, Collection<String> primaryKeys, List<JSONObject> data_list) {
        List<String> cs = cols.stream().map(BaseFieldMeta::getName).collect(Collectors.toList());
        String update = cs.stream()
                .filter(x -> !primaryKeys.contains(x))
                .map(c -> c + " = VALUES(" + c + ")")
                .collect(Collectors.joining(" , "));

        String batch_values = data_list.stream().map(data -> {
            String values = getKeyValue(cols, data);
            return "(" + values + ")";
        }).collect(Collectors.joining(", \n"));
        //String values = getKeyValue(cols, data);
        return "INSERT INTO " + tableName + " (" + StringUtils.join(cs, ",") + ")\n" +
                "VALUES \n" + batch_values + "\n" +
                "ON DUPLICATE KEY UPDATE " + update;
    }

    /**
     * json 转化 sql single
     *
     * @param tableName   表名
     * @param cols        字段值
     * @param primaryKeys 主键
     * @param data        数据
     * @return
     */
    public static String getUpsertSql(String tableName, List<BaseFieldMeta> cols, Collection<String> primaryKeys, JSONObject data) {
        List<String> cs = cols.stream().map(BaseFieldMeta::getName).collect(Collectors.toList());
        String update = cs.stream()
                .filter(x -> !primaryKeys.contains(x))
                .map(c -> c + " = VALUES(" + c + ")")
                .collect(Collectors.joining(" , "));

        String values = getKeyValue(cols, data);
        return "INSERT INTO " + tableName + " (" + StringUtils.join(cs, ",") + ")\n" +
                "VALUES (" + values + ")\n" +
                "ON DUPLICATE KEY UPDATE " + update;
    }

    public static String getKeyValue(List<BaseFieldMeta> cols, JSONObject data) {
        String re = cols.stream().map(c -> {
            String name = c.getName();
            String typeName = c.getOdps_type();
            Object value = getValue(data, name, typeName);
            if (null == value) return null;
            return "'" + value + "'";
        }).collect(Collectors.joining(" , "));
        return re;
    }

    /**
     * 转义sql 值
     *
     * @param data
     * @param name
     * @param typeName
     * @return
     */
    private static Object getValue(JSONObject data, String name, String typeName) {
        Object d;
        switch (typeName) {
            case "BIGINT":
                d = data.getBigInteger(name);
                break;
            case "DOUBLE":
                d = data.getDouble(name);
                break;
            case "BOOLEAN":
                d = data.getBoolean(name);
                break;
            case "DATETIME":
            case "DATE":
            case "TIMESTAMP":
                try {
                    d = data.getTimestamp(name);
                } catch (Exception e) {
                    d = null;
                }
                break;
            case "STRING":
                d = replace(data.getString(name));
                break;
            case "DECIMAL":
                d = data.getBigDecimal(name);
                break;
            case "INT":
            case "TINYINT":
            case "SMALLINT":
                d = data.getInteger(name);
                break;
            case "FLOAT":
                d = data.getFloat(name);
                break;
            default:
                d = replace(data.getString(name));
                break;
        }
        return d;
    }

    public static String replace(String str) {
        if (StringUtils.isBlank(str)) return null;
        return str.replace("\\", "\\\\")
                .replace("'", "''");
    }

    /**
     * 转换odps 值
     *
     * @param data
     * @param name
     * @param typeName
     * @return
     */
    private static Object getValueOdps(JSONObject data, String name, String typeName) {
        Object d;
        switch (typeName) {
            case "BIGINT":
                d = data.getBigInteger(name);
                break;
            case "DOUBLE":
                d = data.getDouble(name);
                break;
            case "BOOLEAN":
                d = data.getBoolean(name);
                break;
            case "DATETIME":
            case "DATE":
            case "TIMESTAMP":
                try {
                    d = data.getString(name);
                } catch (Exception e) {
                    d = null;
                }
                break;
            case "STRING":
                d = data.getString(name);
                break;
            case "DECIMAL":
                d = data.getBigDecimal(name);
                break;
            case "INT":
            case "TINYINT":
            case "SMALLINT":
                d = data.getInteger(name);
                break;
            case "FLOAT":
                d = data.getFloat(name);
                break;
            default:
                d = data.getString(name);
                break;
        }
        return d;
    }


    public static void main(String[] args) {
        DataToStrV3 res = new DataToStrV3();
        res.setup(null);
        String text = "{\"amount\":\"30.0\",\"blicno\":\"\",\"blictype_cn\":\"\",\"capital\":\"[{\\\"amomon\\\":\\\"30万元\\\",\\\"paymet\\\":\\\"货币\\\",\\\"time\\\":\\\"2053-12-31\\\"}]\",\"capital_actual\":\"\",\"company_id\":\"f12c64a6dfe0401124ff2884b84a33d5\",\"company_name\":\"扬州卓联建筑安装劳务有限公司\",\"country_cn\":\"中国\",\"create_time\":\"2023-12-24 18:28:18\",\"deleted\":\"0\",\"holder_id\":\"pa2673337db25ed7136408ece9f4b33bc\",\"holder_name\":\"李燕\",\"holder_type\":\"1\",\"invtype_cn\":\"自然人股东\",\"major_shareholders\":\"1\",\"percent\":\"1.0\",\"rowkey\":\"f12c64a6dfe0401124ff2884b84a33d5_4039fe1b6fec461da63a1cd4632be0e3\",\"update_time\":\"2023-12-25 02:29:39\"}";
        String text1 = "{\"actual_capital_amount\":\"\",\"actual_capital_currency\":\"\",\"approved_time\":\"2021-12-17 00:00:00\",\"base\":\"\",\"business_scope\":\"法律、法规、国务院决定规定禁止的不得经营；法律、法规、国务院决定规定应当许可（审批）的，经审批机关批准后凭许可（审批）文件经营;法律、法规、国务院决定规定无需许可（审批）的，市场主体自主选择经营。（一般项目：白酒制造及销售（除许可业务外，可自主依法经营法律法规非禁止或限制的项目））\",\"cancel_date\":\"\",\"cancel_reason\":\"\",\"cate_first_code\":\"21\",\"cate_second_code\":\"100\",\"cate_third_code\":\"128\",\"city_code\":\"05\",\"company_id\":\"383e5babed67c0fb9b27fc5ad3b8eb4e\",\"company_org_type\":\"个体工商户\",\"company_org_type_new_std\":\"[个体工商户]\",\"company_type\":\"1\",\"county_code\":\"02\",\"crawled_time\":\"2023-12-24 21:45:39\",\"create_time\":\"2023-12-24 21:45:39\",\"credit_code\":\"92520596MA7EBE7210\",\"deleted\":\"0\",\"dims_approved_time\":\"{}\",\"emails\":\"\",\"estiblish_time\":\"2021-12-17 00:00:00\",\"from_time\":\"\",\"history_names\":\"\",\"lat\":\"27.172146\",\"legal_entities\":\"[{\\\"deleted\\\":0,\\\"name\\\":\\\"蒲春荣\\\",\\\"appointment_date\\\":\\\"\\\",\\\"id\\\":\\\"pc2256049434541ffa62a45409fffb5f9\\\",\\\"type\\\":1,\\\"resignation_date\\\":\\\"\\\"}]\",\"lng\":\"105.375662\",\"logo\":\"\",\"name\":\"毕节金海湖新区蒲春荣酒厂\",\"name_alias\":\"蒲春荣\",\"name_en\":\"\",\"new_cid\":\"\",\"org_approved_institute\":\"\",\"org_number\":\"MA7EBE721\",\"parent_company_id\":\"\",\"phones\":\"13984719716\",\"province_code\":\"52\",\"reg_capital\":\"\",\"reg_capital_amount\":\"\",\"reg_capital_currency\":\"\",\"reg_institute\":\"毕节市市场监督管理局高新区分局\",\"reg_location\":\"贵州省毕节市金海湖新区岔河镇岔河村八组\",\"reg_number\":\"520596600167912\",\"reg_status\":\"存续（在营、开业、在册）\",\"reg_status_std\":\"存续\",\"revoke_date\":\"\",\"revoke_reason\":\"\",\"rowkey\":\"383e5babed67c0fb9b27fc5ad3b8eb4e\",\"score\":\"\",\"social_security_staff_num\":\"\",\"tel_info\":\"[{\\\"deleted\\\":\\\"0\\\",\\\"is_phone\\\":1,\\\"tel\\\":\\\"13984719716\\\",\\\"source\\\":\\\"2021年报\\\",\\\"public_time\\\":\\\"2021-01-01 00:00:00\\\"}]\",\"to_time\":\"\",\"update_time\":\"2023-12-25 05:48:40\",\"wechat_public_num\":\"\"}";
        System.out.println(res.evaluate(text1, "company"));
        System.out.println(res.evaluate(text, "company_holder"));

        System.out.println(res.evaluate("{}", "company"));
        System.out.println(res.evaluate(Arrays.asList(text1, text1), "company"));
    }
}
